#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "event.h"

#define BITS_PER_LONG (sizeof(long) * 8)
#define OFF(x)  ((x)%BITS_PER_LONG)
#define LONG(x) ((x)/BITS_PER_LONG)
#define test_bit(bit, array)    ((array[LONG(bit)] >> OFF(bit)) & 1)

int event_init (char *basedir, int num)
{
  char tmpstr[255] = "";

  int event_fd = 0;

  sprintf (tmpstr, "%s/event%d", basedir, num);

  if ((event_fd = open (tmpstr, O_RDONLY | O_NONBLOCK)) < 0)
    return -1;

  return event_fd;
}

int event_get (int event_fd, struct input_event *ev, int count, int tout)
{
  int rd = 0;

  struct timeval timeout;

  fd_set select_set;

  FD_ZERO (&select_set);
  FD_SET (event_fd, &select_set);

  timeout.tv_sec = 0;
  timeout.tv_usec = tout * 1000;        /* ms */
  if (select (event_fd + 1, &select_set, NULL, NULL, &timeout) <= 0)
    return -2;

  rd = read (event_fd, ev, sizeof (struct input_event) * count);

  if (errno && errno != 22)
    return -3;

  if (rd < (int) sizeof (struct input_event))
    return -1;

  return (rd / sizeof (struct input_event));
}

void event_close (int event_fd)
{

/* disable grabbing mode */
  if (event_fd >= 0)
   {
     event_grab (event_fd, 0);
     close (event_fd);
   }

  event_fd = -1;
}

int event_grab (int event_fd, int on)
{
  return (ioctl (event_fd, EVIOCGRAB, on));

}

int event_scan (char *basedir, int start, int end)
{
  int i = 0;

  unsigned long bit[EV_MAX][NBITS (KEY_MAX)];

  int event_fd = 0;

  for (i = start; i <= end; i++)
   {
     if ((event_fd = event_init (basedir, i)) >= 0)
      {
        ioctl (event_fd, EVIOCGBIT (0, EV_MAX), bit[0]);
        if (test_bit (1, bit[0]) && test_bit (17, bit[0]))
         {
           event_close (event_fd);
           return i;
         }
        else
          event_close (event_fd);
      }

   }

  return -1;

}
